<%-- $Id: view_history.jsp 6355 2013-11-26 11:46:42Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.ChangeHistory"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.plugins.util.Parameters"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ChangeHistoryUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="java.util.Date"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%!
	private static final Item itemType = Item.CHANGEHISTORY;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String tab = Values.getString(request.getParameter("tab"), "properties");
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	final Formatter<Date> timeFormatter = FormatterFactory.getDateTimeFormatter(sc);

	final ChangeHistory history = ChangeHistory.getById(dc, itemId);
	final BasicItem changedItem = history.getItem(null);
	final String itemTitle = HTML.encodeTags(changedItem instanceof Nameable ?
			((Nameable)changedItem).getName() : changedItem.toString());
	
	final String changeTime = timeFormatter.format(history.getTime());
	final String title = "Change history -- " + itemTitle + " (" + changeTime + ")";
	%>

	<base:page type="popup" title="<%=title%>" id="view-history">
	<base:head scripts="~history.js" />
	<base:body>
		<h1><%=title%> <base:help helpid="changehistory.view.properties" /></h1>

		<div class="content bottomborder">
			<table class="fullform outlined">
			<tr>
				<th>Item</th>
				<td><%=title%></td>
			</tr>
			<tr>
				<th>Time</th>
				<td><%=changeTime%></td>
			</tr>
			<tr>
				<th>Type</th>
				<td><%=history.getChangeType()%></td>
			</tr>
			<tr>
				<th>Change info</th>
				<td><%=HTML.niceFormat(history.getChangeInfo())%></td>
			</tr>
			<%
			if (sc.hasPermission(Permission.READ, Item.CHANGEHISTORY_VALUES))
			{
				%>
				<tr>
					<th class="subprompt">-old value</th>
					<td><%=HTML.niceFormat(history.getOldValue())%></td>
				</tr>
				<tr>
					<th class="subprompt">-new value</th>
					<td><%=HTML.niceFormat(history.getNewValue())%></td>
				</tr>
				<%
			}
			%>
			<tr>
				<th>Changed by</th>
				<td><%=ChangeHistoryUtil.getUser(dc, history, false, false)%></td>
			</tr>
			<tr>
				<th>Plug-in</th>
				<td><%=ChangeHistoryUtil.getPlugin(dc, history, false, false)%></td>
			</tr>
			<tr>
				<th>Active project</th>
				<td><%=ChangeHistoryUtil.getProject(dc, history, false, false)%></td>
			</tr>
			<tr>
				<th>Job</th>
				<td><%=ChangeHistoryUtil.getJob(dc, history, false, false)%></td>
			</tr>
			<tr>
				<th>Client application</th>
				<td><%=ChangeHistoryUtil.getClient(dc, history, false, false)%></td>
			</tr>
			<tr>
				<th>Session</th>
				<td><%=ChangeHistoryUtil.getSession(dc, history, false, false)%></td>
			</tr>
			<tr class="dynamic">
				<th></th>
				<td></td>
			</tr>
			</table>
		</div>
	
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="close" title="Close" />
		</base:buttongroup>

	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
