/* $Id: annotations.js 6255 2013-03-22 09:50:51Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for the Annotations
	/common/annotations/*

	@author Nicklas
	@version 2.0
*/

var Annotations = function()
{
	var annotations = {};
	var internal = {};
	
	var lastProtocolId = null;
	var editFrameLoaded = false;
	
	var lastParents = null;
	var inheritFrameLoaded = false;
	
	/**
		Automatically load or reload the 'Edit annotations' frame. The
		frame is loaded if it has not previously been loaded, or if
		the protocolId parameter has changed since last time this method
		was called.
	*/
	annotations.autoLoadEditFrame = function(protocolId, subtypeId)
	{
		if (editFrameLoaded && lastProtocolId == protocolId) return;
		lastProtocolId = protocolId;
		editFrameLoaded = true;
		
		var url = App.getRoot() + 'common/annotations/annotate.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('annotate-data', 'item-type');
		url += '&item_id='+Data.get('annotate-data', 'item-id');;
		if (protocolId) url += '&protocol_id='+protocolId;
		if (subtypeId) url += '&subtype_id='+subtypeId;
		frames['annotations'].location.replace(url);
	}
	
	/**
	 	Save all modified annotations to the given form. If
	 	the 'Edit annotations' frame hasn't been loaded, this
	 	call is ignored.
	*/
	annotations.saveModifiedAnnotationsToForm = function(frm)
	{
		if (!editFrameLoaded) return;
		frames['annotations'].Annotate.saveModifiedAnnotationToForm(frm);
	}
	
	/**
		Automatically load or reload the 'Inherit annotations' frame. The
		frame is loaded if it has not previously been loaded, or if
		the parents parameter has changed since last time this method
		was called.
	*/
	annotations.autoLoadInheritFrame = function(parents)
	{
		if (inheritFrameLoaded)
		{
			if (internal.equalArrays(parents, lastParents)) return;
		}
		lastParents = parents;
		inheritFrameLoaded = true;
		
		var url = App.getRoot()+'common/annotations/inherit.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('annotate-data', 'item-type');
		url += '&item_id='+Data.get('annotate-data', 'item-id');
		if (parents && parents.length)
		{
			for (var i = 0; i < parents.length; i++)
			{
				url += '&parents='+parents[i];
			}
		}
		frames['inheritedAnnotations'].location.replace(url);
	}

	/**
	 	Save all changes to inherited annotations to the given form. If
	 	the 'Inherit annotations' frame hasn't been loaded, this
	 	call is ignored.
	*/
	annotations.saveInheritedAnnotationsToForm = function(frm)
	{
		if (!inheritFrameLoaded) return;
		frames['inheritedAnnotations'].Inherit.saveInheritedAnnotationsToForm(frm);
	}
	
	// Check if two arrays have the same elements
	internal.equalArrays = function(a1, a2)
	{
		if (a1.length != a2.length) return false;
		
		for (var i = 0; i < a1.length; i++)
		{
			if (a1[i] != a2[i]) return false;
		}
		
		return true;
	}
	
	return annotations;
}();

