<%-- $Id: preferences.jsp 6529 2014-09-19 09:58:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	This page is used to display and modify the preferences for
	the logged in user. It displays a tabbed dialogue:

	@param page
		The name active page of the tabbed dialog.
		- appearance: Font sizes, scale factor, etc.

	Saving the form invokes the submit_user.jsp page with cmd=SavePreferences

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Metadata"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.util.ToStringComparator"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="net.sf.basedb.util.Values"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.TreeSet"
	import="java.util.HashSet"
	import="java.util.Arrays"
	import="java.util.Iterator"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final String activePage = Values.getStringOrNull(request.getParameter("page"));
final DbControl dc = sc.newDbControl();
try
{
	final User user = User.getById(dc, sc.getLoggedInUserId());
	
	final String longTexts = Values.getString(sc.getUserClientSetting("text.long"), "display");
	
	final boolean hasImages = Values.getBoolean(sc.getUserClientSetting("toolbar.images"), true);
	final boolean hasText = Values.getBoolean(sc.getUserClientSetting("toolbar.text"), true);
	
	final String minColor = Values.getString(sc.getUserClientSetting("ratiocolor.min"), "0000FF");
	final String midColor = Values.getString(sc.getUserClientSetting("ratiocolor.mid"), "FFFFFF");
	final String maxColor = Values.getString(sc.getUserClientSetting("ratiocolor.max"), "FFFF00");
	
	final boolean rememberDialogPositions = Values.getBoolean(sc.getUserClientSetting("dialogs.remember-positions"), true);
	
	ItemContext cc = sc.getCurrentContext(Item.USERCLIENTSETTING);
	List<String> recentColors = cc.getRecent("colors");
	List<String> recentDateFormats = cc.getRecent("dateFormats");
	List<String> recentDateTimeFormats = cc.getRecent("dateTimeFormats");
	
	// Load sticky items
	String mostRecent = Values.getString(sc.getUserClientSetting("menu.mostRecent"),
			"EXPERIMENT:BIOASSAYSET:TRANSFORMATION");
	Set<String> stickyItems = new HashSet<String>(Arrays.asList(mostRecent.split(":")));
	Set<Item> allItems = new TreeSet<Item>(new ToStringComparator<Item>(false));
	allItems.addAll(Metadata.getOwnableItems());
	allItems.add(Item.BIOASSAYSET);
	%>
	<base:page type="popup" title="<%="Preferences for "+HTML.encodeTags(user.getName())%>" id="preferences">
	<base:head scripts="tabcontrol-2.js,~preferences.js" styles="tabcontrol.css">
		<style>
		.colorbox
		{
			margin: 2px;
		}
		</style>		
	</base:head>
	<base:body>
		<h1>Preferences for <%=HTML.encodeTags(user.getName())%> <base:help tabcontrol="settings" /></h1>
		<form action="submit_user.jsp?ID=<%=ID%>" method="post" name="preferences">
		<input type=hidden name="cmd" value="SavePreferences">
		<input type="hidden" name="mincolor" id="mincolor" value="<%=minColor%>">
		<input type="hidden" name="midcolor" id="midcolor" value="<%=midColor%>">
		<input type="hidden" name="maxcolor" id="maxcolor" value="<%=maxColor%>">

		<t:tabcontrol 
			id="settings"
			subclass="content dialogtabcontrol"
			active="<%=activePage%>" remember="<%=activePage == null%>"  	 
			position="bottom">
			<t:tab 
				id="appearance" 
				title="Appearance" 
				helpid="userpreferences.appearance"
				>
				<table class="fullform">
				<tr>
					<th>Font size</th>
					<td>
						<%
						String fontsize = Values.getString(sc.getUserClientSetting("appearance.fontsize"), "size_m.css");
						%>
						<input type="radio" name="fontsize" id="fontsizeXS" value="size_xs.css" 
							<%="size_xs.css".equals(fontsize) ? "checked" : ""%>
							data-scale="80"
							><label for="fontsizeXS">XS</label>
						<input type="radio" name="fontsize" id="fontsizeS" value="size_s.css" 
							<%="size_s.css".equals(fontsize) ? "checked" : ""%>
							data-scale="90"
							><label for="fontsizeS">S</label>
						<input type="radio" name="fontsize" id="fontsizeM" value="size_m.css" 
							<%="size_m.css".equals(fontsize) ? "checked" : ""%>
							data-scale="100"
							><label for="fontsizeM">M</label>
						<input type="radio" name="fontsize" id="fontsizeL" value="size_l.css" 
							<%="size_l.css".equals(fontsize) ? "checked" : ""%>
							data-scale="115"
							><label for="fontsizeL">L</label>
						<input type="radio" name="fontsize" id="fontsizeXL" value="size_xl.css" 
							<%="size_xl.css".equals(fontsize) ? "checked" : ""%>
							data-scale="130"
							><label for="fontsizeXL">XL</label>
					</td>
				</tr>
				<tr>
					<th>Scale factor</th>
					<td>
						<input class="text required" type="text" name="scale" id="scale"
						value="<%=Values.getInt(sc.getUserClientSetting("appearance.scale"), 100)%>" 
						size="4" maxlength="3">%
						(50 - 200)
					</td>
				</tr>
				<tr>
					<th>Display long texts</th>
					<td>
						<input type="radio" name="longTexts" id="longTextsAlways" value="display" 
							<%="display".equals(longTexts) ? "checked" : "" %>
							><label for="longTextsAlways">Always</label>
						<input type="radio" name="longTexts" id="longTextsHover" value="hover" 
							<%="hover".equals(longTexts) ? "checked" : "" %>
							><label for="longTextsHover">On hover</label>
						<input type="radio" name="longTexts" id="longTextsClick" value="click" 
							<%="click".equals(longTexts) ? "checked" : "" %>
							><label for="longTextsClick">On click</label>
					</td>
				</tr>		
				<tr>
					<th>Toolbar</th>
					<td>
						<input type="radio" name="toolbar" id="toolbarBoth" value="both" 
							<%=hasImages && hasText ? "checked" : "" %>
							><label for="toolbarBoth">Images and text</label>
						<input type="radio" name="toolbar" id="toolbarImages" value="images" 
							<%=hasImages && !hasText ? "checked" : "" %>
							><label for="toolbarImages">Images only</label>
						<input type="radio" name="toolbar" id="toolbarText" value="text" 
							<%=!hasImages && hasText ? "checked" : "" %>
							><label for="toolbarText">Text only</label>
					</td>
				</tr>		
				<tr>
					<th>Ratio color range</th>
					<td>
						<span id="btnMinColor" tabindex="0" class="link" data-dialog-title="Color for min value" data-color-box="mincolor"
							><span id="mincolor.visual" style="background: #<%=minColor%>; display: inline-block;" class="colorbox large">&nbsp;</span>Min</span>
						
						<span id="btnMidColor" tabindex="0" class="link" data-dialog-title="Color for midpoint value" data-color-box="midcolor"
							><span id="midcolor.visual" style="background: #<%=midColor%>; display: inline-block;" class="colorbox large">&nbsp;</span>Mid</span>
						
						<span id="btnMaxColor" tabindex="0" class="link" data-dialog-title="Color for max value" data-color-box="maxcolor"
							><span id="maxcolor.visual" style="background: #<%=maxColor%>; display: inline-block;" class="colorbox large">&nbsp;</span>Max</span>
					

							<select name="presets" id="colorPresets" class="selectionlist">
							<option value="">- presets -
							<option value="FF0000,FFFF00,00FF00">Red - Yellow - Green
							<option value="0000FF,FFFFFF,FFFF00">Blue - White - Yellow
							<option value="000000,999999,FFFFFF">Black - Grey - White
							<%
							if (recentColors != null && recentColors.size() > 0)
							{
								%>
								<option value="" class="recentheader" disabled>- recently used -
								<%
								for (int i = 0; i < recentColors.size(); ++i)
								{
									String color = recentColors.get(i);
									%>
									<option value="<%=color%>" class="recent"><%=color%>
									<%
								}
							}
							%>
							</select>
					</td>
				</tr>
				<tr>
					<th>Date format</th>
					<td>
						<input class="text" type="text" name="date_format" 
						value="<%=HTML.encodeTags(FormatterSettings.getDateFormat(sc))%>" 
						size="20">
						<select name="datePresets" id="datePresets" class="selectionlist">
							<option value="">- presets -
							<option value="yyyy-MM-dd">yyyy-MM-dd
							<option value="d/M/yyyy">d/M/yyyy
							<option value="M/d/yyyy">M/d/yyyy
							<%
							if (recentDateFormats != null && recentDateFormats.size() > 0)
							{
								%>
								<option value="" class="recentheader" disabled>- recently used -
								<%
								for (int i = 0; i < recentDateFormats.size(); ++i)
								{
									String format = HTML.encodeTags(recentDateFormats.get(i));
									%>
									<option value="<%=format%>" class="recent"><%=format%>
									<%
								}
							}
							%>
						</select>
					</td>
				</tr>
				
				<tr>
					<th>Date-time format</th>
					<td>
						<input class="text" type="text" name="datetime_format" 
						value="<%=HTML.encodeTags(FormatterSettings.getDateTimeFormat(sc))%>" 
						size="20">
						<select name="dateTimePresets" id="dateTimePresets" class="selectionlist">
							<option value="">- presets -
							<option value="yyyy-MM-dd HH:mm:ss">yyyy-MM-dd HH:mm:ss
							<option value="d/M/yyyy HH:mm:ss">d/M/yyyy HH:mm:ss
							<option value="M/d/yyyy HH:mm:ss">M/d/yyyy HH:mm:ss
							<%
							if (recentDateTimeFormats != null && recentDateTimeFormats.size() > 0)
							{
								%>
								<option value="" class="recentheader" disabled>- recently used -
								<%
								for (int i = 0; i < recentDateTimeFormats.size(); ++i)
								{
									String format = HTML.encodeTags(recentDateTimeFormats.get(i));
									%>
									<option value="<%=format%>" class="recent"><%=format%>
									<%
								}
							}
							%>
						</select>
					</td>
				</tr>
				<%
				int numDecimals = FormatterSettings.getNumDecimals(sc);
				%>
				<tr>
					<th>Decimals</th>
					<td>
						<select name="decimals">
						<%
						for (int i = 0; i < 8; i++)
						{
							%>
							<option value="<%=i%>" <%=numDecimals == i ? "selected" : ""%>><%=i%>
							<%
						}
						%>
						<option value="-1" <%=numDecimals < 0 ? "selected" : ""%>>all
						</select>
					</td>
				</tr>
				<tr>
					<th>Dialogs</th>
					<td>
						<label id="rememberPositions"><input type="checkbox" name="remember_positions" value="1" 
							<%=rememberDialogPositions ? "checked" : "" %>
							>Remember positions</label>
					</td>
				</tr>		
				<tr class="dynamic">
					<th></th>
					<td></td>
				</tr>
				</table>
			</t:tab>
			
			<t:tab
				id="plugins" 
				title="Plugins" 
				helpid="userpreferences.plugins"
				>
				<table class="fullform">
				<tr>
					<th>Messages</th>
					<td>
						<%
						boolean sendMessage = Values.getBoolean(sc.getUserClientSetting("plugins.sendmessage"), true);
						%>
						<input type="checkbox" name="sendmessage" id="sendMessage" value="1" <%=sendMessage ? "checked" : ""%>
							><label for="sendMessage">Send message when plugin completes</label>
					</td>
				</tr>
				<tr>
					<th>Remove jobs</th>			
					<td>
						<%
						boolean removeJobWhenFinished = Values.getBoolean(sc.getUserClientSetting("plugins.removejob"), false);
						%>
						<input type="checkbox" name="removejob" id="removeJob" value="1" <%=removeJobWhenFinished ? "checked" : "" %>
							><label for="removeJob">Remove import and export jobs when they are finished.</label>
					</td>
				</tr>
				<tr>
					<th>Show warnings</th>			
					<td>
						<%
						boolean showWarnings = Values.getBoolean(sc.getUserClientSetting("plugins.showWarnings"), false);
						%>
						<input type="checkbox" name="show_warnings" id="showWarnings" value="1" <%=showWarnings ? "checked" : "" %>
							><label for="showWarnings">Show plug-in warning messages when starting a new job.</label>
					</td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td></td>
				</tr>
				</table>
			</t:tab>
			
			<t:tab
				id="mostRecent"
				title="Recent items"
				helpid="userpreferences.mostrecent"
				tooltip="Specify how many items to remember in lists of 'most recent items'"
				>
				<%
				boolean loadNames = Values.getBoolean(sc.getUserClientSetting("menu.mostRecent.loadNames"), true);
				%>
				<table class="fullform">
				<tr >
					<th>Recently viewed items</th>
					<td>
						<input class="text required" type="text" name="maxViewed" id="maxViewed"
							value="<%=Values.getInt(sc.getUserClientSetting("menu.mostRecent.maxViewed"), 6)%>" 
							size="4" maxlength="2">
						(0 - 10)
					</td>
				</tr>
				<tr>
					<th>Recently used items</th>
					<td>
						<input class="text required" type="text" name="maxUsed" id="maxUsed"
							value="<%=Values.getInt(sc.getUserClientSetting("appearance.recent"), 4)%>" 
							size="4" maxlength="2">
						(0 - 10)
					</td>
				</tr>
				<tr>
					<th>Load item names</th>
					<td>
						<input type="checkbox" name="loadNames" id="loadNames" value="1" <%=loadNames ? "checked" : ""%>>
						<label for="loadNames">Load the names of all items</label>
					</td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td>
				<table>
				<tr>
					<td>
						<base:buttongroup
							vertical="true">
							<base:button
								id="moveUp"
								image="move_up.png"
								tooltip="Move up" 
								subclass="square"
								data-down="0"
							/>
							<base:button 
								id="moveDown"
								image="move_down.png"
								tooltip="Move down" 
								subclass="square"
								data-down="1"
							/>
						</base:buttongroup>
					</td>
					<td>
						<b>Sticky items</b><br>
						<select name="sticky_items" id="stickyItems" multiple size="12" style="width: 12em;">
							<%
							Iterator<Item> it = allItems.iterator();
							while (it.hasNext())
							{
								Item item = it.next();
								if (stickyItems.contains(item.name()))
								{
									it.remove();
									%>
									<option value="<%=item.name()%>"><%=item.toString() %>
									<%
								}
							}
							%>
						</select>
					</td>
					<td>
						<base:buttongroup
							vertical="true"
							>
							<base:button
								id="moveLeft"
								image="move_left.png" 
								tooltip="Show this item in the 'Most recent' menu" 
								subclass="square"
							/>
							<base:button 
								id="moveRight"
								image="move_right.png" 
								tooltip="Don't show this item in the 'Most recent' menu" 
								subclass="square"
							/>
						</base:buttongroup>
					</td>
					<td>
						<b>All items</b><br>
						<select name="all_items" id="allItems" multiple size="12" style="width: 12em;">
							<%
							for (Item item : allItems)
							{
								%>
								<option value="<%=item.name()%>"><%=item.toString() %>
								<%
							}
							%>
						</select>
					</td>
				</tr>
				</table>
					
					</td>
				</tr>
				</table>
			</t:tab>
		</t:tabcontrol>
		</form>

		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
